/*
 * Filename: dip_pixelstack.h
 *
 * (C) Copyright 2004-2005               Pattern Recognition Group
 *     All rights reserved               Faculty of Applied Physics
 *                                       Delft University of Technology
 *                                       Lorentzweg 1
 *                                       2628 CJ Delft
 *                                       The Netherlands
 *
 * Contact: Dr. ir. Lucas J. van Vliet
 * email  : lucas@ph.tn.tudelft.nl
 *
 * Author: Cris Luengo
 */

#ifndef DIP_PIXELSTACK_H
#define DIP_PIXELSTACK_H
#ifdef __cplusplus
extern "C" {
#endif

typedef enum
{
   DIP_GVSO_HIGH_FIRST,
   DIP_GVSO_LOW_FIRST
} dipf_GreyValueSortOrder;

#define DIP_PS_BLOCKSIZE 1024
#define DIP_PH_BLOCKSIZE 65536

typedef struct
{
   void* pointer;               /* pointer into image */
   dip_sfloat value;            /* pixel value - used for sorting */
} dip_PixelStackPixel;

typedef struct dip__tmp__PixelStackBlock
{
   dip_int bottom;              /* pointer to bottom of stack (place to push data onto) */
   dip_int top;                 /* pointer to top of stack (place to pop data from) */
   dip_PixelStackPixel *pixels; /* array with info for each pixel */
   dip_int* coord_data;         /* data block to store coordinates to point into -
                                   coord_data+jj*ndims is a pointer to an array with
                                   ndims elements. */
   struct dip__tmp__PixelStackBlock *next;  /* next block */
} dip__PixelStackBlock, *dip_PixelStackBlock;

typedef struct
{

   /* don't ever change these elements unless you know what you are doing */
   dip_int ndims;               /* dimensionality of coordinate system */
   dip_PixelStackBlock bottom;
   dip_PixelStackBlock top;
   
   /* read-only elements */
   dip_int nelements;           /* keeps track of the number of elements on the stack - not used
                                   by algorithms */
   dip_int blocksize;           /* size of blocks, defaults to DIP_PS_BLOCKSIZE */
   dip_PixelStackBlock marker_block;
   dip_int marker_indx;         /* marker set to bottom at beginning of iteration, when
                                   top==marker_block && top->top==marker_indx, a new iteration
                                   starts - automatically updated when popping the first element
                                   in an iteration - useless and always 0 when sorting!!! */

   /* user-serviceable elements - set these to change behaviour */
   /* note that 'sort' and 'order' should be set before adding data to the stack, they
      control how each pixel is added - weird stuff will happen when changing these
      values with data on the stack */
   dip_Boolean sort;            /* DIP_TRUE if data needs to be sorted upon pushing */
   dipf_GreyValueSortOrder order; /* sorting order, defaults to DIP_GVSO_HIGH_FIRST */

} dip__PixelStack, *dip_PixelStack;

typedef struct dip__tmp__PixelHeapBlock
{
   dip_PixelStackPixel *pixels; /* array with info for each pixel */
   dip_int* coord_data;         /* data block to store coordinates to point into -
                                   coord_data+jj*ndims is a pointer to an array with
                                   ndims elements. */
   struct dip__tmp__PixelHeapBlock *prev;  /* previous block */
   struct dip__tmp__PixelHeapBlock *next;  /* next block */
} dip__PixelHeapBlock, *dip_PixelHeapBlock;

typedef struct
{

   /* don't ever change these elements unless you know what you are doing */
   dip_int ndims;               /* dimensionality of coordinate system */
   dip_PixelHeapBlock firstblock;
   dip_PixelHeapBlock lastblock;
   dip_int nblocks;             /* number of blocks allocated */
   dip_int nelements;           /* number of elements on the stack */
   dip_int blocksize;           /* size of blocks, defaults to DIP_PH_BLOCKSIZE */

   /* user-serviceable elements - set these to change behaviour */
   /* note that 'order' should be set before adding data to the stack, it
      controls how each pixel is added - weird stuff will happen when changing this
      value with data on the heap */
   dipf_GreyValueSortOrder order; /* sorting order, defaults to DIP_GVSO_HIGH_FIRST */

} dip__PixelHeap, *dip_PixelHeap;

DIP_ERROR dip_PixelStackNew ( dip_PixelStack*, dip_int, dip_int, dip_Resources );
DIP_ERROR dip_PixelStackFree ( dip_PixelStack* );    /* defined through a macro */
DIP_ERROR dip_PixelStackPush ( dip_PixelStack, dip_int*, void*, dip_sfloat );
DIP_ERROR dip_PixelStackPop ( dip_PixelStack, dip_int*, void**, dip_sfloat*, dip_Boolean* );
DIP_ERROR dip_PixelStackIsEmpty ( dip_PixelStack, dip_Boolean* );

dip_Boolean dip__PixelStackIsEmpty ( dip_PixelStack ); /* not available outside DIPlib! */

DIP_ERROR dip_PixelHeapNew ( dip_PixelHeap*, dip_int, dip_int, dip_Resources );
DIP_ERROR dip_PixelHeapFree ( dip_PixelHeap* );    /* defined through a macro */
DIP_ERROR dip_PixelHeapPush ( dip_PixelHeap, dip_int*, void*, dip_sfloat );
DIP_ERROR dip_PixelHeapPop ( dip_PixelHeap, dip_int*, void**, dip_sfloat* );
DIP_ERROR dip_PixelHeapIsEmpty ( dip_PixelHeap, dip_Boolean* );

dip_Boolean dip__PixelHeapIsEmpty ( dip_PixelHeap ); /* not available outside DIPlib! */

#ifdef __cplusplus
}
#endif
#endif
