%READTIMESERIES   Reads a series of images from disk in one image. 
%
% SYNOPSIS 
%  out = readtimeseries(basefilename, extension, range, color)
%   
%  basefilename : File name without the running number or full file name
%  extension    : File extension 
%  range        : First and last index to read [first last]
%  color        : Do the files contain color information 'no','yes'
%
% DEFAULTS:
%  extension    : ''
%  range        : [0 0] %all found images are read
%  color        : no 
%
% EXAMPLE:
%  In the directory /data/here, there are 16 files:
%   myfile4.tif,myfile5.tif,myfile006.tif,...myfile020.tif
%  out = readtimeseries('/data/here/myfile','tif')  or equivalent
%  out = readtimeseries('/data/here/myfile012.tif')
%  
% SEE ALSO:
%  readim, readcolorim 
%
% NOTE:
%  For files like myleica_z004_ch01.tif etc. use 
%  readtimeseries('myleica_z*_ch01','tif')
%
%  If you don't want to see 'reading file...' define the global variable
%  SupressOutput = 1

% (C) Copyright 1999-2004               Pattern Recognition Group
%     All rights reserved               Faculty of Applied Physics
%                                       Delft University of Technology
%                                       Lorentzweg 1
%                                       2628 CJ Delft
%                                       The Netherlands
% Bernd Rieger, June 2001.
% March, 2004, renamed D2toD3 to readtimeseries and added more functionality (BR)
% April, 2004, changed filelist to be more flexible, use now find_files 
%              instead of num_slices (BR)
% April, 2004, allowing empty range parameter (I'm lazy!). Removed second reading
%              of first input image (Bernd is lazy too!). Provided for the (unlikely)
%              case that the input slices are not RGB. (CL)
% July 2004,   bug fix for readcolor images, changed default to 'tif' (BR)
% July 2004,   filename is browseable (BR)
% September 2004, moved trimming IN to FIND_FILES.M; changed extension parameter to
%                 string, defaulting to '', changed default file name (CL)

function out=readtimeseries(varargin)

d = struct('menu','File I/O',...
 'display','Read Time Series',...
 'inparams',struct('name',       {'in','ext','range','col'},...
                   'description',{'Filename (base)','Extension','Range','Color information'},...
                   'type',       {'infile','string','array','boolean'},...
                   'dim_check',  {0,0,-1,0},...
                   'range_check',{'*.*','','N',[]},...
                   'required',   {1,0,0,0},...
                   'default',    {'imser','',[0 0],0}...
                    ),...
 'outparams',struct('name',{'out'},...
                    'description',{'Output image'},...
                    'type',{'image'}...
                    )...
 );
if nargin == 1
   s = varargin{1};
   if ischar(s) & strcmp(s,'DIP_GetParamList')
      out = d;
      return
   end
end
try
   [in,ext,range,col] = getparams(d,varargin{:});
catch
   if ~isempty(paramerror)
      error(paramerror)
   else
      error(firsterr)
   end
end
if isempty(range)
   range = [0,0];
elseif prod(size(range))~=2
   error('The range parameter must be of the form [start ending]');
end

fns = find_files(in,ext,range(1),range(2));
M = length(fns);
if M == 0
   error('Found no matching files.');
end

tmp = squeeze(readim(fns{1},ext)); % this also works for color images to get the size, dims
fdim = length(size(tmp));
global SupressOutput
if  isempty(SupressOutput) | ~SupressOutput
   fprintf('Reading %dD file: %s\n',fdim,fns{1});
end
colsp = '';
if col
   tmp = readcolorim(fns{1},ext);
   out = newimar(size(tmp));
   colsp = colorspace(tmp);
   for ii = 1:prod(size(tmp))
      out{ii} = newim([size(tmp{ii}) M],datatype(tmp{ii}));
   end
   ss = ['out(' repmat(':,',1,fdim) 'ii-1) = readcolorim(fns{ii},ext);'];
else
   %make it possible to read big tif series larger than 512x512x1024
   try
      out = newim([size(tmp) M],datatype(tmp));
   catch      
      if strfind(lasterr,'Out of memory.')
         fprintf(['Trying to fool matlab into creating a larger' ...
          ' image than allowed for doubles.\n']);
         out = read2(fns,ext,fdim,M,size(tmp),datatype(tmp));
      else
         error(lasterr);
      end
   end
   ss = ['out(' repmat(':,',1,fdim) 'ii-1) = readim(fns{ii},ext);'];
end
eval (['out(' repmat(':,',1,fdim) '0) = tmp;'])

for ii = 2:M
   %s= [in sprintf(sz0,ii) '.' ext];
   if isempty(SupressOutput) | ~SupressOutput
      fprintf('Reading %dD file: %s\n',fdim,fns{ii});
   end
   eval(ss)
end
if ~isempty(colsp)
   out = colorspace(out,colsp);
end


function out = read2(fns,ext,fdim,M,sz,ddt)
o1 = round(M/2);
o2 = M-o1+1;

fprintf(' Reading part 1.\n');
ss = ['out1(' repmat(':,',1,fdim) 'ii-1) = readim(fns{ii},ext);'];
out1 = newim([sz o1],ddt);
for ii=1:o1
   eval(ss);
end

fprintf(' Reading part 2.\n');
ss = ['out2(' repmat(':,',1,fdim) 'ii-1) = readim(fns{ii},ext);'];
out2 = newim([sz o2],ddt)
for ii=o2:M
   eval(ss);
end
out = cat(fdim,out1,out2);
